@@ -28,7 +28,7 @@ from collections import defaultdict
 
 import dendropy
 
-from .common import is_float
+from gtdbtk.biolib_lite.common import canonical_gid, is_float
 
 """
 To do:
@@ -47,6 +47,14 @@ class Taxonomy(object):
     Spaces after the semi-colons are optional.
     """
 
+    DOMAIN_IDX = 0
+    PHYLUM_IDX = 1
+    CLASS_IDX = 2
+    ORDER_IDX = 3
+    FAMILY_IDX = 4
+    GENUS_IDX = 5
+    SPECIES_IDX = 6
+
     rank_prefixes = ('d__', 'p__', 'c__', 'o__', 'f__', 'g__', 's__')
     rank_labels = ('domain', 'phylum', 'class', 'order',
                    'family', 'genus', 'species')
@@ -783,7 +791,7 @@ class Taxonomy(object):
 
         return taxonomy
 
-    def read(self, taxonomy_file):
+    def read(self, taxonomy_file, canonical_ids=False):
         """Read Greengenes-style taxonomy file.
 
         Expected format is:
@@ -795,12 +803,14 @@ class Taxonomy(object):
         Parameters
         ----------
         taxonomy_file : str
-            Greengenes-style taxonomy file.
+            Path to a Greengenes-style taxonomy file.
+        canonical_ids : bool
+            True if to use the canonical ID format, False otherwise.
 
         Returns
         -------
-        dict : d[unique_id] -> [d__<taxon>, ..., s__<taxon>]
-            Taxa indexed by unique ids.
+        dict[str, tuple[str, str, str, str, str, str, str]]
+            d[unique_id] -> [d__<taxon>, ..., s__<taxon>]
         """
 
         try:
@@ -809,6 +819,8 @@ class Taxonomy(object):
                 for row, line in enumerate(f.readlines()):
                     line_split = line.split('\t')
                     unique_id = line_split[0]
+                    if canonical_ids:
+                        unique_id = canonical_gid(unique_id)
 
                     tax_str = line_split[1].rstrip()
                     if tax_str[-1] == ';':
