@@ -15,12 +15,15 @@
 #                                                                             #
 ###############################################################################
 
-
 import logging
 import os
 import shutil
 import subprocess
 import sys
+import tempfile
+from typing import Dict, Tuple
+
+from tqdm import tqdm
 
 import gtdbtk.config.config as Config
 from gtdbtk.ani_rep import ANIRep
@@ -33,8 +36,12 @@ from gtdbtk.biolib_lite.logger import colour
 from gtdbtk.biolib_lite.taxonomy import Taxonomy
 from gtdbtk.classify import Classify
 from gtdbtk.config.output import *
+from gtdbtk.decorate import Decorate
 from gtdbtk.exceptions import *
 from gtdbtk.external.fasttree import FastTree
+from gtdbtk.infer_ranks import InferRanks
+from gtdbtk.io.batchfile import Batchfile
+from gtdbtk.io.classify_summary import ClassifySummaryFileAR122
 from gtdbtk.markers import Markers
 from gtdbtk.misc import Misc
 from gtdbtk.reroot_tree import RerootTree
@@ -62,12 +69,11 @@ class OptionsParser(object):
         """Check that GTDB-Tk is using the most up-to-date reference package."""
         pkg_ver = float(Config.VERSION_DATA.replace('r', ''))
         min_ver = float(Config.MIN_REF_DATA_VERSION.replace('r', ''))
-        self.logger.info('Using GTDB-Tk reference data version {}: {}'
-                         .format(Config.VERSION_DATA, Config.GENERIC_PATH))
+        self.logger.info(f'Using GTDB-Tk reference data version '
+                         f'{Config.VERSION_DATA}: {Config.GENERIC_PATH}')
         if pkg_ver < min_ver:
-            self.logger.warning(colour('You are not using the reference data '
-                                       'intended for this release: {}'
-                                       .format(Config.MIN_REF_DATA_VERSION),
+            self.logger.warning(colour(f'You are not using the reference data '
+                                       f'intended for this release: {Config.MIN_REF_DATA_VERSION}',
                                        ['bright'], fg='yellow'))
 
     def _verify_genome_id(self, genome_id):
@@ -91,10 +97,10 @@ class OptionsParser(object):
 
         invalid_chars = set('()[],;=')
         if any((c in invalid_chars) for c in genome_id):
-            self.logger.error('Invalid genome ID: %s' % genome_id)
-            self.logger.error(
-                'The following characters are invalid: %s' % ' '.join(invalid_chars))
-            raise GenomeNameInvalid('Invalid genome ID: {}'.format(genome_id))
+            self.logger.error(f'Invalid genome ID: {genome_id}')
+            self.logger.error(f'The following characters are invalid: '
+                              f'{" ".join(invalid_chars)}')
+            raise GenomeNameInvalid(f'Invalid genome ID: {genome_id}')
         return True
 
     def _genomes_to_process(self, genome_dir, batchfile, extension):
@@ -119,46 +125,16 @@ class OptionsParser(object):
         if genome_dir:
             for f in os.listdir(genome_dir):
                 if f.endswith(extension):
-                    genome_id = remove_extension(f)
+                    genome_id = remove_extension(f, extension)
                     genomic_files[genome_id] = os.path.join(genome_dir, f)
 
         elif batchfile:
-            with open(batchfile, "r") as fh:
-                for line_no, line in enumerate(fh):
-                    line_split = line.strip().split("\t")
-                    if line_split[0] == '':
-                        continue  # blank line
+            batchfile_fh = Batchfile(batchfile)
+            genomic_files, tln_tables = batchfile_fh.genome_path, batchfile_fh.genome_tln
 
-                    if len(line_split) not in {2, 3}:
-                        raise GTDBTkExit('Batch file must contain either 2 '
-                                         'columns (detect translation table), '
-                                         'or 3 (specify translation table).')
-
-                    if len(line_split) == 2:
-                        genome_file, genome_id = line_split
-                    elif len(line_split) == 3:
-                        genome_file, genome_id, tln_table = line_split
-                        if tln_table not in {'4', '11'}:
-                            raise GTDBTkExit('Specified translation table must '
-                                             'be either 4, or 11.')
-                        tln_tables[genome_id] = int(tln_table)
-
-                    self._verify_genome_id(genome_id)
-
-                    if genome_file is None or genome_file == '':
-                        raise GTDBTkExit(
-                            'Missing genome file on line %d.' % (line_no + 1))
-                    elif genome_id is None or genome_id == '':
-                        raise GTDBTkExit(
-                            'Missing genome ID on line %d.' % (line_no + 1))
-                    elif genome_id in genomic_files:
-                        raise GTDBTkExit(
-                            'Genome ID %s appears multiple times.' % genome_id)
-                    if genome_file in genomic_files.values():
-                        self.logger.warning(
-                            'Genome file appears multiple times: %s' % genome_file)
-
-                    genomic_files[genome_id] = genome_file
+        # Check that all of the genome IDs are valid.
+        for genome_key in genomic_files:
+            self._verify_genome_id(genome_key)
 
         # Check that the prefix is valid and the path exists
         invalid_paths = list()
@@ -199,36 +175,58 @@ class OptionsParser(object):
 
         return genomic_files, tln_tables
 
-    def _marker_set_id(self, bac120_ms, ar122_ms, rps23_ms):
-        """Get unique identifier for marker set.
+    def _read_taxonomy_files(self, options) -> Dict[str, Tuple[str, str, str, str, str, str, str]]:
+        """Read and merge taxonomy files."""
 
-        Parameters
-        ----------
-        bac120_ms : bool
-        ar122_ms : bool
-        rps23_ms : bool
+        self.logger.info('Reading GTDB taxonomy for representative genomes.')
+        taxonomy = Taxonomy().read(Config.TAXONOMY_FILE)
 
-        Returns
-        -------
-        str
-            The unique identifier for the marker set.
+        if options.gtdbtk_classification_file:
+            # add and overwrite taxonomy for genomes specified in the
+            # GTDB-Tk classification file
+            check_file_exists(options.gtdbtk_classification_file)
 
-        Raises
-        ------
-        GenomeMarkerSetUnknown
-            If the marker set is unknown.
-        """
+            self.logger.info('Reading GTDB-Tk classification file.')
+            gtdbtk_taxonomy = Taxonomy().read(options.gtdbtk_classification_file)
+            del gtdbtk_taxonomy['user_genome']
+            num_reassigned = 0
+            for gid, taxa in gtdbtk_taxonomy.items():
+                if gid in taxonomy:
+                    num_reassigned += 1
+                taxonomy[gid] = taxa
 
-        if bac120_ms:
-            marker_set_id = "bac120"
-        elif ar122_ms:
-            marker_set_id = "ar122"
-        elif rps23_ms:
-            marker_set_id = "rps23"
-        else:
-            self.logger.error('No marker set specified.')
-            raise GenomeMarkerSetUnknown('No marker set specified.')
-        return marker_set_id
+            self.logger.info(f'Read GTDB-Tk classifications for {len(gtdbtk_taxonomy):,} genomes.')
+            self.logger.info(f'Reassigned taxonomy for {num_reassigned:,} GTDB representative genomes.')
+
+        if options.custom_taxonomy_file:
+            # add and overwrite taxonomy for genomes specified in the
+            # custom taxonomy file
+            check_file_exists(options.custom_taxonomy_file)
+
+            self.logger.info('Reading custom taxonomy file.')
+            custom_taxonomy = Taxonomy().read(options.custom_taxonomy_file)
+            num_reassigned = 0
+            for gid, taxa in custom_taxonomy.items():
+                if gid in taxonomy:
+                    num_reassigned += 1
+                taxonomy[gid] = taxa
+
+            self.logger.info(f'Read custom taxonomy for {len(custom_taxonomy):,} genomes.')
+            self.logger.info(f'Reassigned taxonomy for {num_reassigned:,} GTDB representative genomes.')
+
+        if options.gtdbtk_classification_file and options.custom_taxonomy_file:
+            dup_genomes = set(gtdbtk_taxonomy).intersection(custom_taxonomy)
+            if len(dup_genomes) > 0:
+                self.logger.error('GTDB-Tk classification and custom taxonomy '
+                                  'files must not specify taxonomies for the '
+                                  'same genomes.')
+                self.logger.error('These files have {:,} genomes in common.'.format(len(dup_genomes)))
+                self.logger.error('Example duplicate genome: {}'.format(dup_genomes.pop()))
+                raise GTDBTkExit('Duplicated taxonomy information.')
+
+        self.logger.info(f'Read taxonomy for {len(taxonomy):,} genomes.')
+
+        return taxonomy
 
     def identify(self, options):
         """Identify marker genes in genomes.
@@ -247,8 +245,9 @@ class OptionsParser(object):
 
         make_sure_path_exists(options.out_dir)
 
-        genomes, tln_tables = self._genomes_to_process(
-            options.genome_dir, options.batchfile, options.extension)
+        genomes, tln_tables = self._genomes_to_process(options.genome_dir,
+                                                       options.batchfile,
+                                                       options.extension)
         self.genomes_to_process = genomes
 
         markers = Markers(options.cpus)
@@ -256,7 +255,8 @@ class OptionsParser(object):
                          tln_tables,
                          options.out_dir,
                          options.prefix,
-                         options.force)
+                         options.force,
+                         options.write_single_copy_genes)
 
         self.logger.info('Done.')
 
@@ -272,9 +272,6 @@ class OptionsParser(object):
         check_dir_exists(options.identify_dir)
         make_sure_path_exists(options.out_dir)
 
-        if not hasattr(options, 'outgroup_taxon'):
-            options.outgroup_taxon = None
-
         markers = Markers(options.cpus, options.debug)
         markers.align(options.identify_dir,
                       options.skip_gtdb_refs,
@@ -289,7 +286,7 @@ class OptionsParser(object):
                       options.min_perc_taxa,
                       options.out_dir,
                       options.prefix,
-                      options.outgroup_taxon,
+                      options.outgroup_taxon if hasattr(options, 'outgroup_taxon') else None,
                       self.genomes_to_process)
 
         self.logger.info('Done.')
@@ -306,10 +303,7 @@ class OptionsParser(object):
         check_file_exists(options.msa_file)
         make_sure_path_exists(options.out_dir)
 
-        if options.cpus > 1:
-            check_dependencies(['FastTreeMP'])
-        else:
-            check_dependencies(['FastTree'])
+        check_dependencies(['FastTree' + ('MP' if options.cpus > 1 else '')])
 
         if hasattr(options, 'suffix'):
             output_tree = os.path.join(options.out_dir,
@@ -331,7 +325,7 @@ class OptionsParser(object):
 
         fasttree = FastTree()
         fasttree.run(output_tree, tree_log, fasttree_log, options.prot_model,
-                     options.no_support, options.no_gamma, options.msa_file,
+                     options.no_support, options.gamma, options.msa_file,
                      options.cpus)
         self.logger.info(f'FastTree version: {fasttree.version}')
 
@@ -361,46 +355,65 @@ class OptionsParser(object):
             If the test fails.
         """
 
-        make_sure_path_exists(options.out_dir)
+        # Use a temporary directory if none is supplied.
+        if options.out_dir:
+            out_dir_fh = None
+            make_sure_path_exists(options.out_dir)
+        else:
+            out_dir_fh = tempfile.TemporaryDirectory(prefix='gtdbtk_tmp_')
+            options.out_dir = out_dir_fh.name
+            self.logger.info('Using a temporary directory as out_dir was not specified.')
 
-        output_dir = os.path.join(options.out_dir, 'output')
-        genome_test_dir = os.path.join(options.out_dir, 'genomes')
-        if os.path.exists(genome_test_dir):
-            self.logger.error(
-                'Test directory {} already exists'.format(genome_test_dir))
-            self.logger.error('Test must be run in a new directory.')
-            sys.exit(1)
+        try:
+            output_dir = os.path.join(options.out_dir, 'output')
+            genome_test_dir = os.path.join(options.out_dir, 'genomes')
+            if os.path.exists(genome_test_dir):
+                self.logger.error(f'Test directory {genome_test_dir} already exists.')
+                self.logger.error('Test must be run in a new directory.')
+                sys.exit(1)
 
-        current_path = os.path.dirname(os.path.realpath(__file__))
-        input_dir = os.path.join(current_path, 'tests', 'data', 'genomes')
+            current_path = os.path.dirname(os.path.realpath(__file__))
+            input_dir = os.path.join(current_path, 'tests', 'data', 'genomes')
 
-        shutil.copytree(input_dir, genome_test_dir)
+            shutil.copytree(input_dir, genome_test_dir)
 
-        args = ['gtdbtk', 'classify_wf', '--genome_dir', genome_test_dir,
-                '--out_dir', output_dir, '--cpus', str(options.cpus)]
-        self.logger.info('Command: {}'.format(' '.join(args)))
+            args = ['gtdbtk', 'classify_wf', '--genome_dir', genome_test_dir,
+                    '--out_dir', output_dir, '--cpus', str(options.cpus)]
+            self.logger.info('Command: {}'.format(' '.join(args)))
 
-        path_stdout = os.path.join(options.out_dir, 'test_execution.log')
-        with open(path_stdout, 'w') as fh_stdout:
-            proc = subprocess.Popen(args, stdout=fh_stdout,
-                                    stderr=subprocess.PIPE)
-            proc.communicate()
+            # Pipe the output and write to disk.
+            path_stdout = os.path.join(options.out_dir, 'test_execution.log')
+            with subprocess.Popen(args, stdout=subprocess.PIPE,
+                                  stderr=subprocess.PIPE, encoding='utf-8') as proc:
+                with open(path_stdout, 'w') as fh_stdout:
+                    bar_fmt = ' <TEST OUTPUT> '.center(22) + '{desc}'
+                    with tqdm(bar_format=bar_fmt, leave=False) as p_bar:
+                        while True:
+                            line = proc.stdout.readline()
+                            if not line:
+                                break
+                            fh_stdout.write(f'{line}')
+                            p_bar.set_description_str(line.strip())
+                proc.wait()
+                exit_code = proc.returncode
 
-        summary_file = os.path.join(
-            output_dir, PATH_AR122_SUMMARY_OUT.format(prefix='gtdbtk'))
+            summary_fh = ClassifySummaryFileAR122(output_dir, 'gtdbtk')
 
-        if proc.returncode != 0:
-            self.logger.error('The test returned a non-zero exit code.')
-            self.logger.error('A detailed summary of the execution log can be '
-                              'found here: {}'.format(path_stdout))
-            self.logger.error('The test has failed.')
-            sys.exit(1)
-        if not os.path.exists(summary_file):
-            self.logger.error("{} is missing.".format(summary_file))
-            self.logger.error('A detailed summary of the execution log can be '
-                              'found here: {}'.format(path_stdout))
-            self.logger.error('The test has failed.')
-            sys.exit(1)
+            if exit_code != 0:
+                self.logger.error('The test returned a non-zero exit code.')
+                self.logger.error('A detailed summary of the execution log can be '
+                                  'found here: {}'.format(path_stdout))
+                self.logger.error('The test has failed.')
+                sys.exit(1)
+            if not os.path.exists(summary_fh.path):
+                self.logger.error(f"{summary_fh.path} is missing.")
+                self.logger.error('A detailed summary of the execution log can be '
+                                  'found here: {}'.format(path_stdout))
+                self.logger.error('The test has failed.')
+                sys.exit(1)
+        finally:
+            if out_dir_fh:
+                out_dir_fh.cleanup()
 
         self.logger.info('Test has successfully finished.')
         return True
@@ -419,17 +432,19 @@ class OptionsParser(object):
         if options.scratch_dir:
             make_sure_path_exists(options.scratch_dir)
 
-        genomes, _ = self._genomes_to_process(
-            options.genome_dir, options.batchfile, options.extension)
+        genomes, _ = self._genomes_to_process(options.genome_dir,
+                                              options.batchfile,
+                                              options.extension)
 
-        classify = Classify(options.cpus, options.pplacer_cpus)
+        classify = Classify(options.cpus, options.pplacer_cpus, options.min_af)
         classify.run(genomes,
                      options.align_dir,
                      options.out_dir,
                      options.prefix,
                      options.scratch_dir,
                      options.recalculate_red,
-                     options.debug)
+                     options.debug,
+                     options.split_tree)
 
         self.logger.info('Done.')
 
@@ -473,17 +488,12 @@ class OptionsParser(object):
         options : argparse.Namespace
             The CLI arguments input by the user.
         """
-        self.logger.warning("Tree rooting is still under development!")
 
         check_file_exists(options.input_tree)
 
-        if options.custom_taxonomy_file:
-            check_file_exists(options.custom_taxonomy_file)
-            taxonomy = Taxonomy().read(options.custom_taxonomy_file)
-        else:
-            taxonomy = Taxonomy().read(Config.TAXONOMY_FILE)
+        taxonomy = self._read_taxonomy_files(options)
 
-        self.logger.info('Identifying genomes from the specified outgroup.')
+        self.logger.info(f'Identifying genomes from the specified outgroup: {options.outgroup_taxon}')
         outgroup = set()
         for genome_id, taxa in taxonomy.items():
             if options.outgroup_taxon in taxa:
@@ -494,22 +504,50 @@ class OptionsParser(object):
                                   options.output_tree,
                                   outgroup)
 
-        # Symlink to the tree summary file, if not run independently
+        self.logger.info('Done.')
+
+    def decorate(self, options):
+        """Decorate tree with GTDB taxonomy.
+
+        Parameters
+        ----------
+        options : argparse.Namespace
+            The CLI arguments input by the user.
+        """
+
+        check_file_exists(options.input_tree)
+
+        taxonomy = self._read_taxonomy_files(options)
+
+        d = Decorate()
+        d.run(options.input_tree,
+              taxonomy,
+              options.output_tree)
+
+        self.logger.info('Done.')
+
+        # symlink to the decorated tree file, if not run independently
         if hasattr(options, 'suffix'):
             if options.suffix == 'bac120':
-                symlink_f(PATH_BAC120_ROOTED_TREE.format(prefix=options.prefix),
+                symlink_f(PATH_BAC120_DECORATED_TREE.format(prefix=options.prefix),
                           os.path.join(options.out_dir,
-                                       os.path.basename(PATH_BAC120_ROOTED_TREE.format(prefix=options.prefix))))
+                                       os.path.basename(PATH_BAC120_DECORATED_TREE.format(prefix=options.prefix))))
+                symlink_f(PATH_BAC120_DECORATED_TREE.format(prefix=options.prefix) + '-table',
+                          os.path.join(options.out_dir,
+                                       os.path.basename(
+                                           PATH_BAC120_DECORATED_TREE.format(prefix=options.prefix) + '-table')))
             elif options.suffix == 'ar122':
-                symlink_f(PATH_AR122_ROOTED_TREE.format(prefix=options.prefix),
+                symlink_f(PATH_AR122_DECORATED_TREE.format(prefix=options.prefix),
                           os.path.join(options.out_dir,
-                                       os.path.basename(PATH_AR122_ROOTED_TREE.format(prefix=options.prefix))))
+                                       os.path.basename(PATH_AR122_DECORATED_TREE.format(prefix=options.prefix))))
+                symlink_f(PATH_AR122_DECORATED_TREE.format(prefix=options.prefix) + '-table',
+                          os.path.join(options.out_dir,
+                                       os.path.basename(
+                                           PATH_AR122_DECORATED_TREE.format(prefix=options.prefix) + '-table')))
             else:
                 raise GenomeMarkerSetUnknown(
                     'There was an error determining the marker set.')
 
-        self.logger.info('Done.')
-
     def check_install(self):
         """ Verify all GTDB-Tk data files are present.
 
@@ -523,19 +561,16 @@ class OptionsParser(object):
         misc.check_install()
         self.logger.info('Done.')
 
-    def decorate(self, options):
-        """Decorate tree with GTDB taxonomy.
-
-        Parameters
-        ----------
-        options : argparse.Namespace
-            The CLI arguments input by the user.
-        """
+    def infer_ranks(self, options):
+        """Establish taxonomic ranks of internal nodes using RED."""
 
         check_file_exists(options.input_tree)
 
-        # Config.TAXONOMY_FILE
-        self.logger.warning('DECORATE NOT YET IMPLEMENTED!')
+        p = InferRanks()
+        p.run(options.input_tree,
+              options.ingroup_taxon,
+              options.output_tree)
+
         self.logger.info('Done.')
 
     def ani_rep(self, options):
@@ -548,12 +583,13 @@ class OptionsParser(object):
         """
         make_sure_path_exists(options.out_dir)
 
-        genomes, _ = self._genomes_to_process(
-            options.genome_dir, options.batchfile, options.extension)
+        genomes, _ = self._genomes_to_process(options.genome_dir,
+                                              options.batchfile,
+                                              options.extension)
 
         ani_rep = ANIRep(options.cpus)
         ani_rep.run(genomes, options.no_mash, options.mash_d, options.out_dir, options.prefix,
-                    options.mash_k, options.mash_v, options.mash_s, options.min_af)
+                    options.mash_k, options.mash_v, options.mash_s, options.min_af, options.mash_db)
 
         self.logger.info('Done.')
 
@@ -571,7 +607,7 @@ class OptionsParser(object):
             raise GTDBTkExit('Python 2 is no longer supported.')
 
         # Correct user paths
-        if hasattr(options, 'out_dir'):
+        if hasattr(options, 'out_dir') and options.out_dir:
             options.out_dir = os.path.expanduser(options.out_dir)
 
         # Assert that the number of CPUs is a positive integer.
@@ -582,16 +618,16 @@ class OptionsParser(object):
 
         if options.subparser_name == 'de_novo_wf':
             check_dependencies(['prodigal', 'hmmalign'])
-            check_dependencies(
-                ['FastTree' + ('MP' if options.cpus > 1 else '')])
+            check_dependencies(['FastTree' + ('MP' if options.cpus > 1 else '')])
 
+            options.write_single_copy_genes = False
             self.identify(options)
 
             options.identify_dir = options.out_dir
             options.skip_trimming = False
             self.align(options)
 
-            if options.bac120_ms:
+            if options.bacteria:
                 options.suffix = "bac120"
             else:
                 options.suffix = "ar122"
@@ -633,18 +669,34 @@ class OptionsParser(object):
                                                   PATH_AR122_UNROOTED_TREE.format(prefix=options.prefix))
                 options.output_tree = os.path.join(options.out_dir,
                                                    PATH_AR122_ROOTED_TREE.format(prefix=options.prefix))
-            else:
-                self.logger.error(
-                    'There was an error determining the marker set.')
-                raise GenomeMarkerSetUnknown(
-                    'Unknown marker set: {}'.format(options.suffix))
 
             self.root(options)
+
+            if options.suffix == 'bac120':
+                options.input_tree = os.path.join(options.out_dir,
+                                                  PATH_BAC120_ROOTED_TREE.format(prefix=options.prefix))
+                options.output_tree = os.path.join(options.out_dir,
+                                                   PATH_BAC120_DECORATED_TREE.format(prefix=options.prefix))
+            elif options.suffix == 'ar122':
+                options.input_tree = os.path.join(options.out_dir,
+                                                  PATH_AR122_ROOTED_TREE.format(prefix=options.prefix))
+                options.output_tree = os.path.join(options.out_dir,
+                                                   PATH_AR122_DECORATED_TREE.format(prefix=options.prefix))
+
             self.decorate(options)
 
         elif options.subparser_name == 'classify_wf':
-            check_dependencies(
-                ['prodigal', 'hmmalign', 'pplacer', 'guppy', 'fastANI'])
+
+            # TODO: Remove this block once the split_tree function is implemented.
+            if hasattr(options, 'split_tree') and options.split_tree:
+                self.logger.warning('The split tree option is not yet '
+                                    ' supported, overriding value to False.')
+            options.split_tree = False
+
+            check_dependencies(['prodigal', 'hmmalign', 'pplacer', 'guppy',
+                                'fastANI'])
+
+            options.write_single_copy_genes = False
             self.identify(options)
 
             options.identify_dir = options.out_dir
@@ -671,11 +723,22 @@ class OptionsParser(object):
         elif options.subparser_name == 'infer':
             self.infer(options)
         elif options.subparser_name == 'classify':
+
+            # TODO: Remove this block once the split_tree function is implemented.
+            if hasattr(options, 'split_tree') and options.split_tree:
+                self.logger.warning('The split tree option is not yet '
+                                    ' supported, overriding value to False.')
+            options.split_tree = False
+
+            if options.recalculate_red and options.split_tree:
+                raise GTDBTkExit('--split_tree and --recalculate_red are mutually exclusive.')
             self.classify(options)
         elif options.subparser_name == 'root':
             self.root(options)
         elif options.subparser_name == 'decorate':
             self.decorate(options)
+        elif options.subparser_name == 'infer_ranks':
+            self.infer_ranks(options)
         elif options.subparser_name == 'ani_rep':
             self.ani_rep(options)
         elif options.subparser_name == 'trim_msa':
